/**
 * 基础工具类
 */
const BaseUtil = {
  apiHost: "https://langeasy.com.cn",
  imgCdn: "https://img.beingfine.cn",
  LANGEASY_APP_NAME: "langeasy",
  QLISTEN_APP_NAME: "qlisten",
  LEXIS_APP_NAME: "lexis",
  checkAppName() {
    // 检查应用名称,因为lexis可能是langeasyLexis，所以langeasy判断一定要放到最后
    const userAgent = window.navigator.userAgent;
    const checkAppNameList = [
      BaseUtil.QLISTEN_APP_NAME,
      BaseUtil.LEXIS_APP_NAME,
      BaseUtil.LANGEASY_APP_NAME,
    ];
    for (const appName of checkAppNameList) {
      if (new RegExp(appName, "i").test(userAgent)) {
        return appName;
      }
    }
    return "";
  },
  //json转urlParam
  _parseParam(param, key) {
    var paramStr = "";
    if (
      param instanceof String ||
      param instanceof Number ||
      param instanceof Boolean
    ) {
      paramStr += "&" + key + "=" + encodeURIComponent(param);
    } else {
      $.each(param, function (i) {
        var k =
          key == null
            ? i
            : key + (param instanceof Array ? "[" + i + "]" : "." + i);
        paramStr += "&" + this._parseParam(this, k);
      });
    }
    return paramStr.substr(1);
  },
  _jsonToParam(json) {
    return Object.keys(json)
      .map(function (key) {
        return encodeURIComponent(key) + "=" + encodeURIComponent(json[key]);
      })
      .join("&");
  },
  checkIsAndroid() {
    if (window.navigator.userAgent.match(/android/i)) {
      return true;
    }
    return false;
  },
  /**
   * 执行客户端指令
   * @param {String} str
   */
  executeCmd(str) {
    console.log(str);
    if (this.checkIsAndroid()) {
      if (window.jsInterface) {
        window.jsInterface.execute(str);
      }
    } else {
      if (window.webkit) {
        const arr = str.split("&");
        const aCmd = arr[0].split("=");
        const cmd = aCmd[1];
        const extend = str.replace(arr[0] + "&", "");
        window.webkit.messageHandlers[cmd].postMessage(extend);
      } else {
        window.location.href = "js2oc://?" + str;
      }
    }
  },
  /**
   * 以对象方式运行客户端命令，
   * @param {Object} obj
   */
  executeCmdByObj(obj) {
    // 只有iOS不背url中有多个参数会出现&符号需要encodeUri两次
    const isLexis = BaseUtil.checkAppName() === "lexis";
    if (!this.checkIsAndroid() && isLexis) {
      const cmd = Object.keys(obj)
        .map(function (key) {
          return `${key}=${encodeURIComponent(encodeURIComponent(obj[key]))}`;
        })
        .join("&");
      this.executeCmd(cmd);
    } else {
      const cmd = Object.keys(obj)
        .map(function (key) {
          return `${key}=${encodeURIComponent(obj[key])}`;
        })
        .join("&");
      this.executeCmd(cmd);
    }
  },
  /**
   * 推荐执行客户端指令
   * @param {String} cmd
   * @param {Object} obj
   * @returns {Promise}
   */
  executeCmdParam(cmd, obj = {}) {
    obj.cmd = cmd;
    this.executeCmdByObj(obj);
    return new Promise((resolve) => {
      setTimeout(() => {
        resolve();
      }, 100);
    });
  },
};
